LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY vga IS
PORT (	CLOCK_PLL 	: IN std_logic;          --clock from pll
		--KEY 		: IN std_logic_vector(0 downto 0); --RESET 
		RESET	 	: IN std_logic;
		--NIOS_R		: IN std_logic_vector(7 downto 0);
		--NIOS_G		: IN std_logic_vector(7 downto 0);
		--NIOS_B		: IN std_logic_vector(7 downto 0);
		NIOS_RGB	: IN std_logic_vector(15 downto 0);
		VGA_B		: OUT std_logic_vector(9 downto 0); 
		VGA_R		: OUT std_logic_vector(9 downto 0); 
		VGA_G		: OUT std_logic_vector(9 downto 0);
		VGA_X		: OUT std_logic_vector(9 downto 0); 
		VGA_Y		: OUT std_logic_vector(9 downto 0); 
		--VGA_CLK 	: OUT std_logic; 
		VGA_BLANK 	: OUT std_logic; 	--active low		
		VGA_SYNC 	: OUT std_logic;	--active low
		VGA_HS 		: OUT std_logic;
		VGA_VS 		: OUT std_logic);
END vga;


ARCHITECTURE vg OF vga IS
	--test signals
	SIGNAL red, green, blue, zero 				:std_logic_VECTOR(9 DOWNTO 0);
	
	SIGNAL h_sync, v_sync						:STD_LOGIC;
	SIGNAL video_on, video_on_h, video_on_v		:STD_LOGIC;
	--SIGNAL h_count, v_count 					:STD_LOGIC_VECTOR(9 DOWNTO 0);
	
    --SIGNAL line_fin : std_logic;    --variable to determine if a line has finished drawing
    --SIGNAL line_count : integer;     --variable for counting the amount of lines drawn
    --SIGNAL frame_fin : std_logic;   --variable to determine if a frame had finished drawing 
	
	-- Numbers are calculated with 640*480*85Hz referance Altera DE2 user manual
	-- 1/40Mhz = 25ns Numbers in clockcycles 
	CONSTANT FP_h 			: integer := 58;    --Front porch (horizontal)	25
	CONSTANT DI_h 			: integer := 640; 	--Display interval (horizontal)500
	CONSTANT BP_h 			: integer := 79;    --Back porch (horizontal)55
	CONSTANT HSYNC_TIME 	: integer := 58; 	--H_Sync pulse lenght 80
	
	-- numbers in horizontal lines
	CONSTANT FP_v 			: integer := 1; 	--Front porch (vertical lines)
	CONSTANT DI_v 			: integer := 480; 	--Display interval (vertical lines)
	CONSTANT BP_v  			: integer := 25; 	--Back porch (vertical lines)
	CONSTANT VSYNC_LINES	: integer := 3; 	--V_Sync pulse lenght lines 
   
   
	-- Do not change
	CONSTANT h_sync_active 	: integer := FP_h + DI_h + BP_h;
	CONSTANT v_sync_active 	: integer := FP_v + DI_v + BP_v;
	
	
	SIGNAL h_count_ref	: integer;
	SIGNAL v_count_ref 	: integer;
	
	SIGNAL h_count : integer := 0;
	SIGNAL v_count : integer := 0;
	
	SIGNAL h_vga : std_logic_VECTOR(9 DOWNTO 0) := (OTHERS => '0');
	SIGNAL v_vga : std_logic_VECTOR(9 DOWNTO 0) := (OTHERS => '0');
	
	BEGIN
	-- set ref value for on h_count and v_count
	h_count_ref <= FP_h + DI_h + BP_h + HSYNC_TIME;
	v_count_ref <= FP_v + DI_v + BP_v + VSYNC_LINES;
		
	VGA_SYNC <= '1';
	VGA_BLANK <= '1'; -- when blanc set all bits to zero
	
	
	video_on <= video_on_v AND video_on_h;
	
	
	zero <= "0000000000";
	
	
	PROCESS
		BEGIN
		WAIT UNTIL ((clock_PLL'EVENT) AND ( clock_PLL = '1' ));
		
		IF RESET = '0' THEN
			h_count <= 0;
			v_count <= 0;
			video_on_v <= '0';
			video_on_h <= '0';
			--NIOS_INT <= '0';
		
		ELSE
		
		-- for testing
--				IF (KEY(1) = '0') THEN	
--					red <= "1111111100";
--					green <= "0000000000";
--					blue <= "0000000000";
--				ELSIF (KEY(2) = '0') THEN	
--					red <= "0000000000";
--					green <= "1111111100";
--					blue <= "0000000000";
--				ELSIF (KEY(3) = '0') THEN	
--					red <= "0000000000";
--					green <= "0000000000";
--					blue <= "1111111100";
--				ELSE
--					red <= "1111111100";
--					blue <= "1111111100";
--					green <= "1111111100";
--				END IF;
				
			
			--Generate Horizontal and Vertical Timing Signals for Video Signal
			-- H_count counts pixels (640 + extra time for sync signals)
			
			-- Horizontal Count
			
			IF ( h_count = h_count_ref ) THEN
				h_count <= 0;
			ELSE
				h_count <= h_count + 1;
				
			END IF;
			
			--Generate Horizontal Sync Signal using H_count
			IF (( h_count <= h_count_ref) AND (h_count >= h_sync_active )) THEN
				h_sync <= '0';
			ELSE
				h_sync <= '1';
			END IF;
			
			--V_count counts rows of pixels (480 + extra time for sync signals)
			
			-- Vertical counter
			
			-- 20 is added to h_sync_active to insure that h_sync is finish  
			IF ((v_count >= v_count_ref ) AND ( h_count >= (h_sync_active + 20))) THEN 
				v_count <= 0;
			ELSIF ( h_count = (h_sync_active + 20) ) THEN
				v_count <= v_count + 1;
			END IF;
			
			-- Generate Vertical Sync Signal using V_count
			IF ( v_count <= v_count_ref) AND (v_count >= v_sync_active) THEN
				v_sync <= '0';
			ELSE
				v_sync <= '1';
			END IF;
			
			-- Generate Video on Screen Signals for Pixel Data
			IF ( (h_count <= h_sync_active) AND (h_count >= BP_h)) THEN
				video_on_h <= '1';
			ELSE
				video_on_h <= '0';
			END IF;
			
			IF ((v_count <= v_sync_active) AND (v_count >= BP_v)) THEN
				video_on_v <= '1';
			ELSE
				video_on_v <= '0';
			END IF;
			
			-- Put all video signals through DFFs to eliminate
			-- any delays that can cause a blurry image
			-- Turn off RGB outputs when outside video display area
			IF (video_on = '1') THEN
			  h_vga <= h_vga + 1;
				v_vga <= v_vga + 1;
				VGA_R <= (NIOS_RGB(15 downto 11) & "00000"); --red;
				VGA_G <= (NIOS_RGB(10 downto 5) & "0000"); --green;
				VGA_B <= (NIOS_RGB(4 downto 0) & "00000"); --blue;
			ELSE
			  h_vga <= (OTHERS => '0');
				v_vga <= (OTHERS => '0');
				VGA_R <= zero;
				VGA_G <= zero;
				VGA_B <= zero;
			END IF;
			
			VGA_X	<= h_vga;
			VGA_Y	<= v_vga;
			VGA_HS 	<= h_sync;
			VGA_VS 	<= v_sync;
			
		
			
		END IF;
	END PROCESS;
end vg;